/**
 * @file midirecv.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3 
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and 
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * Receives and translates midi data
**/
#include "midirecv.h"
#include "miditable.h"
#include "../cmdbuf.h"
#include "../dmxstr.h"
#include "../executor.h"
#include <stdio.h>
#include <pthread.h>
#include <alsa/asoundlib.h>

/* translation table for midi */
midi_table_t *ttable = 0;
/* termination flag */
int terminate = 0;
/* learn data */
midib learn[3];
/* midi receive mutex */
pthread_mutex_t midi_mutex;


/**
 * Initiate the midi receiver
**/
void midireceiver_init()
{
	/* init midi data table */
	ttable = miditable_init();
	learn[0] = 0; learn[1] = 0; learn[2] = 0;
	/* init mutex */
	pthread_mutex_init(&midi_mutex, NULL);
}



/**
 * Add a translation
 * @param byte1   Midi byte 1
 * @param byte2   Midi byte 2
 * @param cmd     A text command
**/
void midireceiver_add_translation(midib byte1, midib byte2, const char *cmd)
{
	if(ttable == 0) return;
	if(cmd == 0)    return;	
	/* Add to table */
	pthread_mutex_lock(&midi_mutex);
		miditable_add(ttable, byte1 , byte2, cmd);
	pthread_mutex_unlock(&midi_mutex);
}



/**
 * Add a translation by learned value
 * @param cmd     A text command
**/
void midireceiver_add_translation_l(const char *cmd)
{
	if(ttable == 0) return;
	if(cmd == 0)    return;	
	/* Add to table */
	pthread_mutex_lock(&midi_mutex);
		miditable_add(ttable, learn[0], learn[1], cmd);
	pthread_mutex_unlock(&midi_mutex);
}



/**
 * Initiate the midi receiver
**/
void midireceiver_destroy()
{
	/* tell threads to terminate */
	terminate = 1;
	/* destroy midi data table */
	if(ttable != 0)
		miditable_init(ttable);
}



/**
 * Timer main thread
**/
void *midireceiver_device_thread(void *ptr)
{
	int ret, cnt;
	snd_rawmidi_t *midiin = NULL;
	char portname[100];
	char cmd_buf[512];
	const char *cmd;
	midib buf[3];
	midib in;
	/* get copy of portname */
	strcpy(portname, ptr);
	/* connect midi device */
	if( (ret = snd_rawmidi_open(&midiin, NULL, portname, SND_RAWMIDI_SYNC)) < 0 )
	{
		cmdbuf_printf("MIDI THREAD: Can not open midi device '%s'!", portname);
		cmdbuf_printf("             %s", snd_strerror(ret));
		return NULL;
	}
	/* run */
	cnt = 0;
	while(!terminate)
	{
		if( (ret = snd_rawmidi_read(midiin, &in, 1)) < 0 )
		{
			cmdbuf_printf("MIDI THREAD: Problem reading at '%s'!", portname);
			cmdbuf_printf("             %s", snd_strerror(ret));
			snd_rawmidi_close(midiin);
			return NULL;
		}
		if(terminate) break;
		/* catch status byte */
		if(in > 127) cnt = 0;
		if(cnt < 3)  buf[cnt] = in;
		cnt++;
		if(cnt >= 3)
		{
			pthread_mutex_lock(&midi_mutex);
				cmd = miditable_get_cmd(ttable, buf[0], buf[1]);
				/*cmdbuf_printf("MIDI THREAD: Read [%x,%x,%x]", buf[0], buf[1], buf[2]);*/
				if(cmd != 0)
				{
					dmxstr_parse_value(cmd_buf, cmd, buf[2]);
					executor(cmd_buf, 0);
				}
				learn[0] = buf[0];
				learn[1] = buf[1];
				learn[2] = buf[2];
			pthread_mutex_unlock(&midi_mutex);
		}
	}
	/* close device */
	snd_rawmidi_close(midiin);
	return NULL;
}



/**
 * Add midi device
 * @param portname   Device string (if null hw:1,0,0)
 * @return 1 if successfull
**/
int midireceiver_add_midi_device(const char *portname)
{
	int ret;
	pthread_t thread;
	/* set standart portname */
	if(portname == 0)       portname = "hw:1,0,0";
	if(portname[0] == '\0') portname = "hw:1,0,0";
	/* start device thread */
	ret = pthread_create(&thread, NULL, midireceiver_device_thread, (void*) portname);
	return (ret == 0);
}


