/**
 * lindmx.c
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3 
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and 
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
**/
#include "lindmx.h"
#include <stdio.h>
#include <string.h>
#include <ftdi.h>
#include "config.h"

/* ERROR MESSAGES */
const char *error_msg_noerror  = "No error occurred.";
const char *error_msg_ftdi     = "The FTDI driver is not available.";
const char *error_msg_device   = "The DMX4ALL interface is not available. Ensure that it is plugged in and you have the rights to establish the connection.";
const char *error_msg_ptr      = "Invalid pointer for LINDMX-Object.";
const char *error_msg_channels = "LINDMX received an illegal number of channels.";
const char *error_msg_send     = "Error while sending data to the interface.";
const char *error_msg_recv     = "Error while receiving data from the interface.";
const char *error_msg_unex     = "An unexpected error occurred!";

/* LINDMX DATA OBJECT */
struct lindmx_object {
	int    error;
	int    channels;
	int    simulate;
	struct ftdi_context interface;
} lindmx_object_t;




/**
 * lindmx_get_lib_version()
 * @return The libraries exact build version as string
**/
const char *lindmx_get_lib_version()
{
	return LINDMX_VERSION_STRING;
}



/**
 * lindmx_get_err_str()
 * @param objptr Pointer to the LINDMX object
 * @return String describing the last error that occoured
**/
const char *lindmx_get_err_str(LINDMX objptr)
{	
	if(objptr == 0) return error_msg_ptr;
	struct lindmx_object* obj;
	obj = objptr;
	/* switch the error code */
	switch(obj->error)
	{
		case 0: 
			return error_msg_noerror;
		case 1: 
			return error_msg_ftdi;
		case 2: 
			return error_msg_device;
		case 3: 
			return error_msg_ptr;
		default:
			/* unknown or unexpected */
			return error_msg_unex;
	}
	/* unreachable */
	return error_msg_unex;
}



/**
 * lindmx_get_err()
 * @param objptr Pointer to the LINDMX object
 * @return Error code, 0 if no error
**/
int lindmx_get_err(LINDMX objptr)
{	
	if(objptr == 0) return 3;
	struct lindmx_object* obj;
	obj = objptr;
	return obj->error;
}



/**
 * getFormatedStr()
 * Creates a command-string to send a value for a given channel
 * @param channel  The channel to set
 * @param value    The value to set to
 * @param buf      Must be min. 9 chars long! -> Here the data is written
 * @return Should be 8, 0=NO DATA, -1=FAIL (length of written data)
**/
int getFormatedStr(int channel, int value, unsigned char *buf)
{
	/* char counter */
	int cnt = 0;
	/* check if channel&value is valid */
	buf[0] = '\0';
	if( (channel < 0) || (channel > 511) ) return -1;
	if( (value   < 0) || (value   > 255) ) return -1;
	/* channel */
	buf[cnt++] = 'C';
	buf[cnt++] = '0'+(channel/100)%10;
	buf[cnt++] = '0'+((channel/10)%10);
	buf[cnt++] = '0'+(channel%10);
	/* value */
	buf[cnt++] = 'L';
	buf[cnt++] = '0'+(value/100)%10;
	buf[cnt++] = '0'+((value/10)%10);
	buf[cnt++] = '0'+(value%10);
	/* set end and return length */
	buf[cnt+1] = '\0';
	return cnt;
}



/**
 * getFormatedStrChannels()
 * Creates a command-string to set the number of channels to work with
 * @param channel  The channel to set
 * @param buf      Must be min. 9 chars long! -> Here the data is written
 * @return Should be 8, 0=NO DATA, -1=FAIL (length of written data)
**/
int getFormatedStrChannels(int channels, unsigned char *buf)
{
	/* char counter */
	int cnt = 0;
	/* check if channel&value is valid */
	buf[0] = '\0';
	if( (channels < 0) || (channels > 511) ) return -1;
	/* channel cnt */
	buf[cnt++] = 'N';
	buf[cnt++] = '0'+(channels/100)%10;
	buf[cnt++] = '0'+((channels/10)%10);
	buf[cnt++] = '0'+(channels%10);
	/* set end and return length */
	buf[cnt+1] = '\0';
	return cnt;
}



/**
 * lindmx_create()
 * Creates a new LINDMX-Object to work with
 * THIS NEEDS ROOT RIGHTS! (cause of libftdi)
 * @param channels Maximum number of channels - reduce them for better stability, max. 512
 * @return Pointer to the LINDMX object
**/
LINDMX lindmx_create(int channels)
{
	/* Vars */
	unsigned char cbuf[10];
	int sl, written;
	struct lindmx_object* obj;
	struct ftdi_context*  interface;
	/* Make a lindmx_object */
	obj = malloc(sizeof(struct lindmx_object));
	if(obj == NULL)	return NULL;
	/* Set last-error-value */
	obj->error = 0;
	/* non-simulating */
	obj->simulate = 0;
	/* Get the FTDI-Context struct */
	interface = &(obj->interface);
	/* Init FTDI */
	if(ftdi_init(interface) < 0)
	{
		obj->error = 1;
		return NULL;
	}
	/* Open FTDI-USB-Interface */
	if(ftdi_usb_open(interface, 0x0403, 0xc850) < 0) 
	{
		obj->error = 2;
		return NULL;
	}
	/* Set FTDI-Baudrate 3840 */
	ftdi_set_baudrate(interface, 38400);
	/* Set Channels */
	sl = getFormatedStrChannels(channels, cbuf);
	obj->channels = channels;
	if(sl<=0) 
	{
		ftdi_usb_close(interface);
		ftdi_deinit(interface);
		free(obj);
		return NULL;
	}
	written  = ftdi_write_data(interface, cbuf, sl);
	if(written<sl) 
	{
		ftdi_usb_close(interface);
		ftdi_deinit(interface);
		free(obj);
		return NULL;
	}
	/* Recv */
	ftdi_read_data(interface, cbuf, 12);
	/* Return created object */
	return obj;
}


/**
 * lindmx_create_simulator()
 * Creates a lindmx simulator with console output
 * @param channels Maximum number of channels - reduce them for better stability, max. 512
 * @return Pointer to the LINDMX object
**/
LINDMX lindmx_create_simulator(int channels)
{
	struct lindmx_object* obj;
	/* Make a lindmx_object */
	obj = malloc(sizeof(struct lindmx_object));
	if(obj == NULL)	return NULL;
	/* Set last-error-value */
	obj->error = 0;
	/* non-simulating */
	obj->simulate = 1;
	return obj;
}


/**
 * lindmx_destroy()
 * Destruction-function for the DMX4ALL-Linux-Driver-Object
 * @param objptr Pointer to the LINDMX object
**/
void lindmx_destroy(LINDMX objptr)
{
	/* Test if obj is valid */
	if(objptr == NULL) return;
	/* Get object */
	struct lindmx_object* obj = objptr;
	if(obj->simulate) 
	{
		free(obj);
		return;
	}
	/* Get ftdi-context */
	struct ftdi_context* interface = &(obj->interface);
	if(interface == NULL) return;
	/* Close & Deinit */
	if(interface != NULL) ftdi_usb_close(interface);
	if(interface != NULL) ftdi_deinit(interface);
	/* Kill main object */
	free(obj);
}



/**
 * lindmx_set()
 * Sets a DMX-Channel to a given value
 * @param channel The number of the channel to set
 * @param value   The value to set this channel to
 * @param objptr  The LINDMX pointer
 * @return -1 FAIL, 0 NO DATA, 1 OK
**/
int lindmx_set(int channel, int value, LINDMX objptr)
{
	/* Buffer&Vars */
	unsigned char buf[16];
	int sl, written;
	/* Test if obj is valid */
	if(objptr == NULL) return -1;
	if(((struct lindmx_object*) objptr)->simulate) 
	{
		sl = getFormatedStr(channel, value, buf);
		printf("%s\n", buf);
		return 1;
	}
	/* Get ftdi-context */
	struct ftdi_context* interface = &(((struct lindmx_object*) objptr)->interface);
	if(interface == NULL) return -1;
	/* Get Formated String */
	sl = getFormatedStr(channel, value, buf);
	if(sl<=0) return sl;
	/* Send */
	written = ftdi_write_data(interface, buf, sl);
	if(written<sl) return -1;
	/* Recv */
	ftdi_read_data(interface, buf, 12);
	return written;
}



/**
 * lindmx_set_all()
 * Sets all DMX-Channels to a given value
 * @param value   The value to set all channel to
 * @param objptr  The LINDMX pointer
 * @return -1 FAIL, 0 NO DATA, 1 OK
**/
int lindmx_set_all(int value, LINDMX objptr)
{
	int written, channels, n;
	written = 1;
	channels = ((struct lindmx_object*) objptr)->channels;
	for(n=0; n<=channels; n++)
	{
		if( lindmx_set(n, value, objptr) <= 0 ) written = -1;
	}
	return written;
}



/**
 * lindmx_reset_channels()
 * Sets all DMX-Channels to zero
 * @param objptr  The LINDMX pointer
 * @return -1 FAIL, 0 NO DATA, 1 OK
**/
int lindmx_reset_channels(LINDMX objptr)
{
	return lindmx_set_all(0, objptr);
}
