/**
 * @file confreader.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * Config parsing for lindmx
**/
#include "confreader.h"
#include <stdio.h>
#include <stdlib.h>



/**
 * @function confreader_init
 * Initiates a config reader object for a config file
 * @param file    The config files path for fopen
 * @return Pointer to an confreader_t or NULL on error
**/
confreader_t *confreader_init(const char *file)
{
	/* create and allocate new reader object */
	confreader_t *cr;
	cr = malloc(sizeof(confreader_t));
	/* initialize values */
	cr->filename         = file;
	cr->line             = 0;
	cr->option_buffer[0] = '\0';
	/* try to open file */
	cr->fp = fopen(file, "r");
	/* error opening file */
	if( cr->fp == NULL )
	{
		free(cr);
		return NULL;
	}
	/* ready. */
	return cr;
}



/**
 * @function confreader_destroy
 * Cleans up a config reader object
 * @param cr     Pointer to a config reader object
**/
void confreader_destroy(confreader_t *cr)
{
	/* check cr */
	if( cr == NULL ) return;
	/* close file */
	if( cr->fp != NULL ) fclose(cr->fp);
	cr->fp = NULL;
	/* free memory */
	free(cr);
}



/**
 * @function confreader_getNextOption
 * Looks for the next option
 * @param valuebuf   Buffer where the setting is written
 * @param buflen     Length of the buffer
 * @param cr         Pointer to a config reader object
 * @return  Pointer to the options name or NULL on file end
**/
const char *confreader_getNextOption(char *valuebuf, int buflen, confreader_t *cr)
{
	int n, n_cnt, v_cnt;
	char linebuf[CONFREADER_BUFLEN];
	/* check cr & fp */
	if( cr     == NULL ) return NULL;
	if( cr->fp == NULL ) return NULL;
	/* loop until command was read, skip comments */
	while(1)
	{
		/* check for file end */
		if(feof( cr->fp )) return NULL;
		/* read the line */
		if( fgets(linebuf, CONFREADER_BUFLEN, cr->fp) == NULL )
			return NULL;
		linebuf[CONFREADER_BUFLEN-1] = '\0';
		/* skip whitespace */
		n = 0;
		while( (linebuf[n] == ' ') || (linebuf[n] == '\t') ) n++;
		/* check for comments */
		if( linebuf[n] != '#' && linebuf[n] != '\n' && linebuf[n] != '\0' )
			break;
	}
	/* read option name */
	n_cnt = 0;
	while( (linebuf[n] != ' ') && (linebuf[n] != '\t') )
	{
		cr->option_buffer[n_cnt++] = linebuf[n++];
		if(n_cnt >= (CONFREADER_BUFLEN-1)) break;
	}
	cr->option_buffer[n_cnt++] = '\0';
	/* skip whitespace */
	while( (linebuf[n] == ' ') || (linebuf[n] == '\t') ) n++;
	/* read option value */
	v_cnt = 0;
	while( (linebuf[n] != '\n') && (linebuf[n] != '\n') && (linebuf[n] != '\r') )
	{
		valuebuf[v_cnt++] = linebuf[n++];
		if(v_cnt >= (buflen-1)) break;
	}
	valuebuf[v_cnt++] = '\0';
	/* return option name */
	return cr->option_buffer;
}
