/**
 * @file main.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ---------------------------------------------------------
 * (c) Florian Schmitt / Nov-2012 / All rights reserved. 
 * ---------------------------------------------------------
 * Here the main service is done.
**/
#include "lindmxd.h"
#include "modules/dmxmodule.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>


/* global variables */
int g_maxchannel = 0;
int connections  = 0;
int stopsrv      = 0;
int srv_sock     = 0;
int memory[512];



/**
 * kill_lineend()
 * Kills the line end. The incoming string format does not matter: 
 * '\r', '\n', '\r\n' and '\n\r' is accepted.
 * @param str    String to modify
 * @param buflen buffer length, maximum place to go to
 * @return string length
**/
int kill_lineend(char *str, int buflen)
{
	int n = 0;
	/* look for the line/string end */
	while( n<buflen && (str[n] != '\r') && (str[n] != '\n') && (str[n] != '\0') ) ++n;
	/* place a \0 at the end */
	str[n] = '\0';
	return (n-1);
}




/**
 * lindmxd_thread_client()
 * The client thread
 * @param arg Pointer on the client socket
**/
void *lindmxd_thread_client(void *arg)
{
	int cli_sock = *(int *)arg;
	char buf[LINDMXD_BUFLEN];
	connections++;
	/* Main loop until break or until error while reading/writing */
	int n = 1;
	int chan, val, ret, m;
	while(n > 0)
	{
		/* Receive message from client */
		n = read(cli_sock, buf, LINDMXD_BUFLEN);
		if(n<=0) break;
		/* Server was already down */
		if( stopsrv ) 
		{
			n = write(cli_sock, "out of service\r\n", 16);
			break;
		}
		/* transform \r\n ending to \0 */
		kill_lineend(buf, LINDMXD_BUFLEN);

		/* Server shut-down/stop wanted */
		if( strcmp("stop", buf) == 0 ) 
		{
			/* close client connection */
			sprintf(buf, "going to stop now. bye.\r\n");
			n = write(cli_sock, buf, strlen(buf));
			close(cli_sock);
			/* stop the server */
			lindmxd_stop();
			break;
		}
		/* Client wants to quit */
		else if( (strcmp("quit", buf) == 0) || (strcmp("exit", buf) == 0) ) 
		{
			n = write(cli_sock, "bye\r\n", 6);
			break;
		}
		/* DMX: Full Off */
		else if(strcmp("fulloff", buf) == 0) 
		{
			/* Set memory */
			for(m=0; m<g_maxchannel; m++) memory[m] = 0;
			/* Set device */
			ret = dmxmodule_reset_channels();
			/* check for error */
			if(ret <= 0) 
			{
				syslog(LOG_ERR, "[ERROR] reset all channels failed!");
				n = write(cli_sock, "error\r\n", 7);
			}
			else n = write(cli_sock, "ok\r\n", 4);
		}
		/* DMX: Full On */
		else if(strcmp("fullon", buf) == 0) 
		{
			/* Set memory */
			for(m=0; m<g_maxchannel; m++) memory[m] = 255;
			/* Set device */
			ret = dmxmodule_set_all(255);
			/* check for error */
			if(ret <= 0) 
			{
				syslog(LOG_ERR, "[ERROR] set all channels failed!");
				n = write(cli_sock, "error\r\n", 7);
			}
			else n = write(cli_sock, "ok\r\n", 4);
		}
		/* DMX: Set every channel */
		else if(sscanf(buf, "set * %d", &val) == 1) 
		{
			if((val < 0) || (val>255))
			{
				/* invalid value message */
				sprintf(buf, "Invalid value - range is 0-255\r\n");
			}
			else
			{
				/* Set memory */
				for(m=0; m<g_maxchannel; m++) memory[m] = val;
				/* Set device */
				ret = dmxmodule_set_all(val);
				/* check for error */
				if(ret <= 0) 
				{
					syslog(LOG_ERR, "[ERROR] set all channels failed!");
					sprintf(buf, "error\r\n");
				}
				else sprintf(buf, "ok\r\n");
			}
			/* send answer */
			n = write(cli_sock, buf, strlen(buf));
		}
		/* DMX: Set channel value */
		else if(sscanf(buf, "set %d %d", &chan, &val) == 2) 
		{
			chan--;
			if((chan < 0) || (chan>=g_maxchannel))
			{
				/* invalid channel message */
				sprintf(buf, "Invalid channel - range is 1-%d\r\n", g_maxchannel);
			}
			else if((val < 0) || (val>255))
			{
				/* invalid value message */
				sprintf(buf, "Invalid value - range is 0-255\r\n");
			}
			else
			{
				/* set channel value and give answer */
				memory[chan] = val;
				ret = dmxmodule_set(chan, val);
				/* check for error */
				if(ret <= 0)
				{
					syslog(LOG_ERR, "[ERROR] set channel failed!");
					sprintf(buf, "error - interface unreachable\r\n");
				}
				else sprintf(buf, "ok\r\n");
			}
			/* send answer */
			n = write(cli_sock, buf, strlen(buf));
		}
		/* DMX: get channel value */
		else if(sscanf(buf, "get %d", &chan) == 1) 
		{
			chan--;
			if((chan < 0) || (chan>=g_maxchannel))
			{
				/* invalid channel message */
				sprintf(buf, "Invalid channel - range is 1-%d\r\n", g_maxchannel);
			}
			else
			{
				/* get channel value and give answer */
				sprintf(buf, "%d\r\n", memory[chan]);
			}
			/* send answer */
			n = write(cli_sock, buf, strlen(buf));
		}
		/* Connection counter requested */
		else if(strcmp("connections", buf) == 0) 
		{
			sprintf(buf, "%d active connections\r\n", connections);
			n = write(cli_sock, buf, strlen(buf));
		}
		/* Connection counter requested */
		else if(strcmp("commands", buf) == 0) 
		{
			sprintf(buf, "commands: quit / fullon / fulloff / setall <value> / set <channel> <value> / get <channel> / stop / connections\r\n");
			n = write(cli_sock, buf, strlen(buf));
		}
		/* other messages, unknown */
		else
		{
			sprintf(buf, "command not known - type 'commands' for info\r\n");
			n = write(cli_sock, buf, strlen(buf));
		}
	}

	/* decrease connection counter */
	connections--;
	/* Close client socket */
	close(cli_sock);

	return NULL;
} 



/**
 * lindmxd_run()
 * This will start and run the LINDMX-Daemon
 * @param port        Port to run the server on
 * @param maxchannel  Number of the highest DMX channel
 * @param device      Name of the device to use
 * @param zero        if zero is set, all channels are reset on startup
 * @param ihost       The Interface's Hostname
 * @param iport       The Interface's Port
*/
void lindmxd_run(int port, int maxchannel, const char *interface, int zero, const char *ihost, int iport)
{
	/* Server socket */
	int cli_sock, n;
	struct sockaddr_in srv_addr, cli_addr;

	/* cleanup memory */
	for(n=0; n<g_maxchannel; n++) memory[n] = 0;

	g_maxchannel = maxchannel;
	/* Error while creating the interface object (e.g. no root rights, no interface found) */
	if( !dmxmodule_init(interface, g_maxchannel, ihost, iport) ) 
	{	
     	syslog(LOG_ERR, "CAN NOT INIT DMX INTERFACE");
		exit(EXIT_FAILURE);
	}

	/* Set all channels to zero, if wanted */
	if(zero)
		dmxmodule_reset_channels();


	/* Create server socket */
	srv_sock = socket(AF_INET, SOCK_STREAM, 0);
	if(srv_sock < 0) 
	{
     	syslog(LOG_ERR, "[ERROR] creating the servers socket!");
		exit(EXIT_FAILURE);
	}

	/* Initialize server socket address */
	bzero((char *) &srv_addr, sizeof(srv_addr));
	srv_addr.sin_family       = AF_INET;
	srv_addr.sin_addr.s_addr  = INADDR_ANY;
	srv_addr.sin_port         = htons(port);

	/* Bind address to server socket */
	if( bind(srv_sock, (struct sockaddr *) &srv_addr, sizeof(srv_addr)) < 0 )
	{
		close(srv_sock);
     	syslog(LOG_ERR, "[ERROR] not able to bind address to the server socket!");
		exit(EXIT_FAILURE);
	}

	/* main loop */
	while(!stopsrv)
	{
		/* thread for the client connection */
		pthread_t child_thread;
		/* begin listening */
		listen(srv_sock, 5);

		/* get the new client */
		socklen_t clilen = sizeof(cli_addr);
		cli_sock = accept(srv_sock, (struct sockaddr *) &cli_addr, &clilen);
		if(cli_sock < 0)
		{
		 	syslog(LOG_ERR, "[ERROR] accepting a client");
			continue;
		}

		/* create child thread */
		pthread_create(&child_thread, NULL, lindmxd_thread_client, (void*) &cli_sock);
		/* kill resources when quit / return value not needed */
		pthread_detach(child_thread);
	}

	/* end */
	close(srv_sock);
	dmxmodule_destroy();
	syslog(LOG_NOTICE, "lindmxd shut down successfully");
	exit(EXIT_SUCCESS);
}



/**
 * lindmxd_stop()
 * This function stops the server.
*/
void lindmxd_stop()
{
	/* set stop flag */
	stopsrv = 1;
	/* wait a bit */
	usleep(500*1000);
	/* close server socket and quit */
	close(srv_sock);
	/* clear interface */
	dmxmodule_destroy();
	usleep(500);
	/* final shutdown */
	syslog(LOG_NOTICE, "lindmxd shut down successfully.");
	exit(EXIT_SUCCESS);
}



