/**
 * @file main.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * This file starts the daemon and handles program options.
**/
#include "lindmxd.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <unistd.h>
#include <syslog.h>
#include <signal.h>
#include <sys/stat.h>
#include "config.h"
#include "signals.h"
#include "confreader.h"

const char *program_name;




/**
 * start_daemon()
 * Start the daemon, fork processes etc.
 * @param daemon_name Name of the daemon, also of the logfile
 * @param no_fork     If nofork is set to 1, no fork is done
**/
void start_daemon(const char *daemon_name, int no_fork)
{
	int pid;

	if(no_fork != 1)
	{
		/* fork the processes */
		pid = fork();
		/* something went wrong */
		if(pid < 0) exit(EXIT_FAILURE);
		/* this is the parent process */
		if(pid > 0) exit(EXIT_SUCCESS);

		/* set the child as session leader / returns session id */
		if(setsid() == -1)
		{
			fprintf(stderr, "%s error becoming the session leader.\n", daemon_name);
			exit(0);
		}
	}

	/* set umask to get full file access */
	umask(0);

	/* Goto root directory */
	if( chdir("/") < 0 )
	{
		fprintf(stderr, "Can not toggle daemon to root directory.\n");
		exit(EXIT_FAILURE);
	}

	/* Set log message importance */
	setlogmask( LOG_UPTO(LOG_NOTICE) );

	/* close standart file descriptors */
	if(no_fork != 1)
	{
		/* open syslog */
		openlog(daemon_name, LOG_DAEMON, LOG_LOCAL1);
		close(STDIN_FILENO);
		close(STDOUT_FILENO);
		close(STDERR_FILENO);
	}
	else
	{
		/* open syslog */
		openlog(daemon_name, LOG_DAEMON|LOG_PERROR, LOG_LOCAL1);
		syslog(LOG_NOTICE, "starting lindmxd in no_fork/debug mode");
	}
}


/**
 * print_usage()
 * Prints the program usage
 * @param stream     IOSTREAM to print to
 * @param exit_code  The code to give to exit() after printing
**/
void print_usage(FILE* stream, int exit_code)
{
	fprintf(stream, "Usage:  %s [options]\n", program_name);
	fprintf(stream,
			"  -h  --help            Display this usage information\n"
			"  -v  --version         Display program version\n"
			"  -p  --port n          Set the port to n\n"
			"  -c  --channels n      Set maximum number of channels to n\n"
			"  -i  --interface type  Select interface type\n"
			"  -l  --list            Prints a list of available interface types\n"
			"  -d  --debug           Disable fork and give debug messages\n"
			"  -z  --zero            Set all channels to zero on start-up\n");
	/* additional information */
	fprintf(stream, "\nThis  service is logged by syslog. Watch your logs to fix problems.\n");
	exit(exit_code);
}



/**
 * print_interface_list()
 * Prints a list of available interfaces
 * @param stream     IOSTREAM to print to
 * @param exit_code  The code to give to exit() after printing
**/
void print_interface_list(FILE* stream, int exit_code)
{
	fprintf(stream, "Available interfaces:\n");
	fprintf(stream,
			"   Name:                  |   Description: \n"
			"   -----------------------+----------------------------------------\n"
			"   simulate               |   Simulates an interface\n"
			"   dmx4all_micro_usb      |   DMX4ALL Micro USB Interface\n"
			"   -----------------------+----------------------------------------\n");
	exit(exit_code);
}



/**
 * print_version()
 * Prints the program version
**/
void print_version()
{
	printf("%s: Version %d.%d.%d-%d\n", program_name, LINDMX_MAJOR_VERSION,
	                                                  LINDMX_MINOR_VERSION,
	                                                  LINDMX_PATCH_VERSION,
	                                                  LINDMX_BUILD_NUMBER);
	printf("Copyright © 2013 Florian Schmitt\n"
	       "License GPLv3+: GNU GPL Version 3 or higher <http://gnu.org/licenses/gpl.html>\n"
	       "This program comes with ABSOLUTELY NO WARRANTY.\n"
	       "This is free software, and you are welcome to \n"
	       "redistribute it under certain conditions.\n\n"
	       "This software is written by Florian Schmitt <florian.schmitt@qikgrp.com>\n");
	exit(EXIT_SUCCESS);
}



/**
 * main()
 * Main procedure
**/
int main(int argc, char* argv[])
{
	int next_option, channels, port, debug, zero, if_port;
	char interface[CONFREADER_BUFLEN];
	char confbuf[CONFREADER_BUFLEN];
	char if_host[CONFREADER_BUFLEN];
	const char *option;
	confreader_t *cr;

	/* options */
	const char* const short_options = "hvp:c:si:ldz";
	const struct option long_options[] = {
		{ "help",     0, NULL, 'h' },
		{ "version",  0, NULL, 'v' },
		{ "port",     1, NULL, 'p' },
		{ "channels", 1, NULL, 'c' },
		{ "simulate", 0, NULL, 's' },
		{ "interface",1, NULL, 'i' },
		{ "list",     0, NULL, 'l' },
		{ "debug",    0, NULL, 'd' },
		{ "zero",     0, NULL, 'z' },
		{ NULL,       0, NULL,  0  }
	};

	/* default values */
	program_name = argv[0];
	channels  = 100;
	port      = 6767;
	debug     = 0;
	zero      = 0;
	if_port   = 0;
	strncpy(interface, "dmx4all_micro_usb", CONFREADER_BUFLEN);
	strncpy(if_host, "localhost", CONFREADER_BUFLEN);

	/* read config file */
	cr = confreader_init( LINDMXD_CONFIG );
	/* ignore config on error */
	if(cr != NULL)
	{
		while( (option = confreader_getNextOption(confbuf, CONFREADER_BUFLEN, cr)) != NULL )
		{
			if( strcmp(option, "interface") == 0 )
			{
				strncpy(interface, confbuf, CONFREADER_BUFLEN);
			}
			else if( strcmp(option, "debug") == 0 )
			{
				if( strcmp(confbuf, "true") == 0 ) debug = 1;
			}
			else if( strcmp(option, "zero") == 0 )
			{
				if( strcmp(confbuf, "true") == 0 ) zero = 1;
			}
			else if( strcmp(option, "port") == 0 )
			{
				port = atoi(confbuf);
			}
			else if( strcmp(option, "interface_port") == 0 )
			{
				if_port = atoi(confbuf);
			}
			else if( strcmp(option, "interface_host") == 0 )
			{
				strncpy(if_host, confbuf, CONFREADER_BUFLEN);
			}
			else if( strcmp(option, "channels") == 0 )
			{
				channels = atoi(confbuf);
			}
			else
			{
				fprintf(stderr, "[ERROR] Invalid config command: %s\n", option);
			}
		}
	}
	else fprintf(stderr, "[ERROR] Config file '%s' not found!\n", LINDMXD_CONFIG);

	/* get the options */
	do {
		next_option = getopt_long(argc, argv, short_options, long_options, NULL);
		switch(next_option)
		{
			case 'h':
				print_usage(stdout, EXIT_SUCCESS);
			case 'p':
				port = atoi(optarg);
				break;
			case 'c':
				channels = atoi(optarg);
				break;
			case 'd':
				debug = 1;
				break;
			case 's':
				printf("Simulation mode is deprecated. See --help for info.\n");
				printf("Use 'simulate' as interface name.\n");
				exit(EXIT_FAILURE);
				break;
			case 'i':
				strncpy(interface, optarg, CONFREADER_BUFLEN);
				break;
			case 'l':
				print_interface_list(stdout, EXIT_SUCCESS);
			case 'v':
				print_version();
				break;
			case 'z':
				zero = 0;
				break;
			case '?':
				print_usage(stderr, EXIT_FAILURE);
				break;
			case -1:
				break;
			default:
				abort();
		}
	} while(next_option != -1);

	/* start up daemon */
	if(debug == 1) printf("Starting lindmxd on port %d providing %d DMX channels.\nInterface type: '%s' [DEBUG=ON]\n", port, channels, interface);
	else
	{
		printf("Starting lindmxd on port %d providing %d DMX channels.\nInterface type: '%s'\n", port, channels, interface);
		printf("Being silent now. See syslog for errors or use -d\n");
	}
	start_daemon("lindmxd", debug);

	/* write config error again, to reach log */
	if(cr == NULL) syslog(LOG_ERR, "[ERROR] Config file '%s' not found!\n", LINDMXD_CONFIG);

	/* Set signal destination */
	signal(SIGINT, signals_handle_sig);

	/* run the main servers loop */
	lindmxd_run(port, channels, interface, zero, if_host, if_port);

	/* after running, close log */
	closelog();
	/* clean up config reader */
	confreader_destroy(cr);
 	return 0;
}
