/**
 * @file modules/dmxmodule.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ---------------------------------------------------------
 * (c) Florian Schmitt / Nov-2012 / All rights reserved. 
 * ---------------------------------------------------------
 * Implements interfaces
**/
#include "dmxmodule.h"
#include "stdio.h"
#include <string.h>
#include <syslog.h>
/* Include interface headers */
#include "dmx4all.h"
#include "dmx4all_tcp.h"
#include "dmx4all_udp.h"

unsigned int use_interface;

/* Interface list */
const char *interfaces[] = {
		"simulate",
		"dmx4all_micro_usb",
		"dmx4all_tcp",
		"dmx4all_udp",
		"" /* marks the end */
	};


/**
 * dmxmodule_init
 * Initiates the interface module
 * @param id     Interface id -> See dmxmodule.h definitions
 * @param ifip   Interface IPv4 Address (null for non-networking interfaces)
 * @param port   Interface Network Port (0 for non-networking interfaces)
 * @return 1 on success, 0 on failure
**/
int dmxmodule_init(const char *interface, int maxchannel, const char *ifip, int port)
{
	/* set interface */
	while( 1 )
	{
		/* reached list end */
		if( interfaces[use_interface][0] == '\0' )
		{
     		syslog(LOG_ERR, "[ERROR] Interface '%s' not found in list", interface);
			return 0;
		}
		/* check interface name */
		if( strcmp(interface, interfaces[use_interface]) == 0 )
			break;
		/* increase */
		use_interface++;
	}

	/* switch interface init method */
	switch( use_interface )
	{

		/* Simulate the interface */
		case DMXMODULE_SIMULATE:
			printf("Interface simualtion started.\n");
			break;

		/* DMX4ALL Micro USB */
		case DMXMODULE_DMX4ALL:
			if( !dmx4all_init(maxchannel) )
				return 0;
			break;

		/* DMX4ALL TCP */
		case DMXMODULE_DMX4ALL_TCP:
			if( !dmx4all_tcp_init(maxchannel, ifip, port) )
				return 0;
			break;

		/* DMX4ALL UDP */
		case DMXMODULE_DMX4ALL_UDP:
			if( !dmx4all_udp_init(maxchannel, ifip, port) )
				return 0;
			break;

		/* Invalid interface */
		default:
			/* failure */
     		syslog(LOG_ERR, "[ERROR] Invalid interface type (id=%d)", use_interface);
			return 0;
	}

	/* return success */
	return 1;
}



/**
 * dmxmodule_destroy
 * Cleans up the interface module
**/
void dmxmodule_destroy()
{
	/* switch interface init method */
	switch( use_interface )
	{

		/* Simulate the interface */
		case DMXMODULE_SIMULATE:
			printf("Interface simualtion quit.\n");
			break;

		/* DMX4ALL Micro USB */
		case DMXMODULE_DMX4ALL:
			dmx4all_destroy();
			break;

		/* DMX4ALL TCP */
		case DMXMODULE_DMX4ALL_TCP:
			dmx4all_tcp_destroy();
			break;

		/* DMX4ALL UDP */
		case DMXMODULE_DMX4ALL_UDP:
			dmx4all_udp_destroy();
			break;

		/* Invalid interface */
		default:
     		syslog(LOG_ERR, "[ERROR] Invalid interface type (id=%d)", use_interface);
			return;
	}
}



/**
 * dmxmodule_reset_channels()
 * Sets all channels to 0
 * @return 1 on success, 0 on failure
**/
int dmxmodule_reset_channels()
{
	/* switch interface init method */
	switch( use_interface )
	{

		/* Simulate the interface */
		case DMXMODULE_SIMULATE:
			printf("Reset all channels.\n");
			break;

		/* DMX4ALL Micro USB */
		case DMXMODULE_DMX4ALL:
			if( !dmx4all_reset_channels() )
				return 0;
			break;

		/* DMX4ALL TCP */
		case DMXMODULE_DMX4ALL_TCP:
			if( !dmx4all_tcp_reset_channels() )
				return 0;
			break;

		/* DMX4ALL UDP */
		case DMXMODULE_DMX4ALL_UDP:
			if( !dmx4all_udp_reset_channels() )
				return 0;
			break;

		/* Invalid interface */
		default:
			/* failure */
     		syslog(LOG_ERR, "[ERROR] Invalid interface type (id=%d)", use_interface);
			return 0;
	}

	/* return success */
	return 1;
}



/**
 * dmxmodule_set_all()
 * Set all channels to a given value
 * @param value   Value to set all channels on
 * @return 1 on success, 0 on failure
**/
int dmxmodule_set_all(int value)
{
	/* switch interface init method */
	switch( use_interface )
	{

		/* Simulate the interface */
		case DMXMODULE_SIMULATE:
			printf("set * to %d\n", value);
			break;

		/* DMX4ALL Micro USB */
		case DMXMODULE_DMX4ALL:
			if( !dmx4all_set_all(value) )
				return 0;
			break;

		/* DMX4ALL TCP */
		case DMXMODULE_DMX4ALL_TCP:
			if( !dmx4all_tcp_set_all(value) )
				return 0;
			break;

		/* DMX4ALL UDP */
		case DMXMODULE_DMX4ALL_UDP:
			if( !dmx4all_udp_set_all(value) )
				return 0;
			break;

		/* Invalid interface */
		default:
			/* failure */
     		syslog(LOG_ERR, "[ERROR] Invalid interface type (id=%d)", use_interface);
			return 0;
	}

	/* return success */
	return 1;
}



/**
 * dmxmodule_set()
 * Set a channel to a given value
 * @param channel  Channel to set
 * @param value    Value to set to
 * @return 1 on success, 0 on failure
**/
int dmxmodule_set(int channel, int value)
{
	/* switch interface init method */
	switch( use_interface )
	{

		/* Simulate the interface */
		case DMXMODULE_SIMULATE:
			printf("set %d to %d\n", (channel+1), value);
			break;

		/* DMX4ALL Micro USB */
		case DMXMODULE_DMX4ALL:
			if( !dmx4all_set(channel, value) )
				return 0;
			break;

		/* DMX4ALL TCP */
		case DMXMODULE_DMX4ALL_TCP:
			if( !dmx4all_tcp_set(channel, value) )
				return 0;
			break;

		/* DMX4ALL UDP */
		case DMXMODULE_DMX4ALL_UDP:
			if( !dmx4all_udp_set(channel, value) )
				return 0;
			break;

		/* Invalid interface */
		default:
			/* failure */
     		syslog(LOG_ERR, "[ERROR] Invalid interface type (id=%d)", use_interface);
			return 0;
	}

	/* return success */
	return 1;
}
